from typing import TypedDict

from pydantic import BaseModel

from castom_type.base import DefTypeDict


class SS(BaseModel):
    port: int
    host: str


class Sett(TypedDict):
    port: int
    host: str


class Sett2(DefTypeDict):
    port: int = 7070
    host: str = "555"


if __name__ == '__main__':
    s = Sett(port=23, host=123)
    sw = Sett2(port=23, host=33333)
    sw2 = Sett2(port=23, host=33333)
    sw3 = Sett2(port=23, host=33333)
    print(sw)
    print(sw2)
    print(sw3)



from os.path import splitext
from typing import Any, Union, Iterator

from yaml import load, Loader, dump, load_all
from yaml.composer import ComposerError

from .base_file import BaseFile


class YamlFile(BaseFile):
    def appendFile(self, arg: Any):
        # TODO: appendFile !!!
        pass

    def __init__(self, name_file: str):
        if splitext(name_file)[1] != ".yaml":  # Проверяем расширение файла
            raise ValueError("Файл должен иметь расширение .yaml")

        BaseFile.__init__(self, name_file)

    def readFile(self,
                 *,
                 encoding: str = "utf-8",
                 _Loader=Loader,
                 limit: int = 0
                 ) -> Union[list, dict, Any]:
        """
        @param _Loader: Разрешение чтение данных, которые хранятся в касторном типе. :param encoding: Кодировка.
        @param limit: Если в файле есть документ, то можно указать сколько документов нужно прочитать. -1 пропитать
        все документы из файла
        @param encoding:
        """

        def __read_all(__yamlFile):
            _reader: Iterator = load_all(_yamlFile, Loader=_Loader)
            return [_x for _x in _reader]

        with open(self.name_file, "r", encoding=encoding) as _yamlFile:
            _res: Union[list, dict, Any] = []

            if limit == 0:  # Если нужно прочитать весь файл, в котором нет документов.

                try:
                    _res = load(_yamlFile, Loader=_Loader)
                except ComposerError:  # Если в файле есть несколько документов, то читаем все документы.
                    _yamlFile.seek(0)  # Так как мы уже прочитали весь файл, то нам нужно вернутся в начало файла.
                    _res = __read_all(_yamlFile)

            elif limit > 0:  # Если в файле есть документы, и нам нужно прочитать определенное количество документов.
                reader = load_all(_yamlFile, Loader=_Loader)
                for _index, _row in enumerate(reader):
                    if _index < limit:
                        _res.append(_row)
                    else:
                        break
            else:  # Если в файле есть документы и нам нужно прочитать все документы.
                _res = __read_all(_yamlFile)

        return _res

    def writeFile(self,
                  data: Union[list, dict, Any],
                  *,
                  encoding: str = "utf-8",
                  default_flow_style: bool = False,
                  allow_unicode=True,
                  ):
        """
        @param data:
        @param encoding: Кодировка
        @param default_flow_style: 	Если вы хотите, чтобы коллекции всегда сериализовались
        в блочном стиле установите False
        @param allow_unicode:Экранировать символы, если True данные запишутся как есть.
        """
        with open(self.name_file, "w", encoding=encoding) as _yamlFile:
            dump(data, _yamlFile, default_flow_style=default_flow_style, allow_unicode=allow_unicode)



"""

"""

from typing import Callable

from .sync_base_sql import SyncBaseSql
from .sync_serializer import Efetch

try:
    from sync_mysql.connector import connect, Error
except ModuleNotFoundError:
    pass


class Config(SyncBaseSql):

    @property
    def CONNECT(self):
        return connect

    @property
    def ERROR(self):
        return Error

    def __init__(self, user: str, password: str, dbname: str | None = None,
                 port: int = 3306,
                 host: str = "localhost"):
        super().__init__(user, password, host)
        self.SETTINGS_DB.update({
            "port": port,
            "dbname": dbname,
        })

    def mutable_command(self, _connection,
                        execute: str,
                        params: tuple | dict | list = (),
                        autocommit: bool = False,
                        *args,
                        **kwargs):
        """
        Декоратор для выполнения изменяемой SQL команды
        """
        with _connection.cursor() as cursor:
            cursor.execute(execute, params, multi=kwargs["multi"])
            _connection.commit()
        return cursor.statement

    def read_command(self, _connection,
                     execute: str,
                     params: tuple | dict | list = (),
                     tdata: Callable = Efetch.all,
                     *args,
                     **kwargs):
        """
        Декоратор для выполнения чтения из БД
        """
        with _connection.cursor() as cursor:
            cursor.execute(execute, params, multi=kwargs["multi"])
            return tdata(cursor)



from os import environ, getenv
from pprint import pprint
from re import sub

from .txt_file import TxtFile


class EnvFile(TxtFile):

    def __init__(self, name_file: str, type_file: str = '.env'):
        super().__init__(name_file, type_file=type_file)

        self.IS_READ_ENV_FILE: bool = False

    def readAndSetEnv(self):
        """
        Чтение переменных окружения из указанного файла, и добавление их в ПО `python`
        """
        with open(self.name_file, 'r', encoding='utf-8') as _file:
            res = {}
            for line in _file:
                tmp = sub(r'^#[\s\w\d\W\t]*|[\t\s]', '', line)
                if tmp:
                    k, v = tmp.split('=', 1)
                    # Если значение заключено в двойные кавычки, то нужно эти кавычки убрать
                    if v.startswith('"') and v.endswith('"'):
                        v = v[1:-1]

                    res[k] = v
        environ.update(res)
        pprint(res)
        self.IS_READ_ENV_FILE = True

    def getEnv(self, name_env: str) -> str:
        """
        Взять переемную окружения, если мы не читали файл с переменными окружения, то вызовется ошибка
        """
        if not self.IS_READ_ENV_FILE:
            raise FileExistsError("Вы не прочитали файл с переменными окружения")
        return getenv(name_env)



from collections import namedtuple
from enum import Enum
from typing import Callable, Any


def dictfetchall(cursor) -> list[dict[str, Any]]:
    """
    Вернуть в виде словаря
    """
    columns = [col[0] for col in cursor.description]
    return [
        dict(zip(columns, row))
        for row in cursor.fetchall()
    ]


def namedtuplefetchall(cursor) -> list[namedtuple]:
    """
    Вернуть в виде именовано го картежа
    """
    desc = cursor.description
    nt_result = namedtuple('_', [col[0] for col in desc])
    return [nt_result(*row) for row in cursor.fetchall()]


class Efetch(Enum):
    dict_: Callable[[Any], list[dict[str, Any]]] = dictfetchall
    namedtuple: Callable[[Any], list[namedtuple]] = namedtuplefetchall
    one: Callable[[Any], Any] = lambda _cursor: _cursor.fetchone()
    all: Callable[[Any], Any] = lambda _cursor: _cursor.fetchall()



import unittest
from os.path import getsize

from mg_file.file.txt_file import TxtFile


class TestFile(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        # Имя фалйа
        self.name_file = "test.txt"
        # Данные для теста
        self.test_str: str = "ninja cjj,output На двух языках 1#1^23 !23№эЭ123'"

    # Этот метод запускается ПЕРЕД каждой функции теста
    def setUp(self) -> None:
        self.testClassFile = TxtFile(self.name_file)
        self.testClassFile.deleteFile()
        self.testClassFile.createFileIfDoesntExist()

    def test_sizeFile(self):
        # Проверка определение размера файла
        self.testClassFile.writeFile(self.test_str)
        self.assertEqual(self.testClassFile.sizeFile(), getsize(self.testClassFile.name_file))

    def test_deleteFile_and_checkExistenceFile(self):
        # Проверка удаление файла
        self.assertEqual(self.testClassFile.checkExistenceFile(), True)
        self.testClassFile.deleteFile()
        self.assertEqual(self.testClassFile.checkExistenceFile(), False)

    def test_writeFile(self):
        # Проверка записи в файл
        self.testClassFile.writeFile(self.test_str)
        self.assertEqual(self.test_str, self.testClassFile.readFile())

    def test_appendFile(self):
        # Проверка дозaписи в файл
        test_str: str = self.test_str
        self.testClassFile.writeFile(test_str)
        self.testClassFile.appendFile(test_str)
        test_str += test_str
        self.assertEqual(test_str, self.testClassFile.readFile())

    def test_readBinaryFile_and_writeBinaryFile(self):
        # Проверка записи и чтения в двоичном режиме
        self.testClassFile.writeBinaryFile(self.test_str.encode())
        self.assertEqual(self.test_str.encode(), self.testClassFile.readBinaryFile())

    def test_appendBinaryFile(self):
        # Проверка до записи в двоичном режиме
        tests: str = self.test_str
        self.testClassFile.writeBinaryFile(tests.encode())
        self.testClassFile.appendBinaryFile(tests.encode())
        tests += tests
        self.assertEqual(tests.encode(), self.testClassFile.readBinaryFile())

    def test_readFile_Line(self):
        test_text = "123123\n3123133\n12312d1d12313"
        self.testClassFile.writeFile(test_text)
        self.assertEqual(self.testClassFile.readFile(2), "123123\n3123133\n")

    def test_readFileToResDict(self):
        self.testClassFile.writeFile("my name\nmy passwd\nmy token")
        res = self.testClassFile.readFileToResDict("name", "passwd", "token")
        self.assertEqual(res, {'name': 'my name', 'passwd': 'my passwd', 'token': 'my token'})

    def test_searchFile(self):
        test_text = "Optional. If the number of \n bytes returned exceed the hint number, \n no more lines will be returned. Default value is  -1, which means all lines will be returned."
        self.testClassFile.writeFile(test_text)
        self.assertEqual(self.testClassFile.searchFile("more"), True)

    def test___init__QuackCommand(self):
        TxtFile("test.txt", mod="w", data="123123")
        r1 = TxtFile("test.txt", mod="r")
        self.assertEqual(r1.res, "123123")
        TxtFile("test.txt", mod="a", data="99")
        r2 = TxtFile("test.txt", mod="r")
        self.assertEqual(r2.res, "12312399")

    def __del__(self):
        self.testClassFile.deleteFile()


class test_File(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.name_file = "test/test/data/test.txt"

    def test_route(self):
        self.testClassFile = TxtFile(self.name_file)
        self.testClassFile.writeFile("123")
        self.assertEqual(self.testClassFile.readFile(), "123")
        self.testClassFile.removeRoute()


if __name__ == '__main__':
    unittest.main()



import importlib.util
from abc import abstractmethod
from hashlib import sha256
from importlib.machinery import ModuleSpec
from os import makedirs, remove, mkdir
from os.path import abspath, dirname, exists, getsize, splitext
from pathlib import Path
from shutil import rmtree
from types import ModuleType
from typing import Any, Callable, TypeAlias, Union, Optional

T_ConcatData: TypeAlias = Union[list[Union[str, int, float]], list[list[Union[str, int, float]]], dict, set, str]


class BaseFile:
    """
    Базовый класс для файлов
    """
    __slots__ = "name_file"

    def __init__(self, name_file: str, type_file: str, mod: str = None):
        self.check_extensions_file(name_file, type_file)
        self.name_file: str = name_file
        self.createFileIfDoesntExist()

    @staticmethod
    def check_extensions_file(name_file: str, req_type: str):
        """
        Проверить расширение файла

        @param name_file: Путь к файлу
        @param req_type: Требуемое расширение
        """
        if splitext(name_file)[1] != req_type:  # Проверяем расширение файла
            raise ValueError(f"Файл должен иметь расширение {req_type}")

    def createFileIfDoesntExist(self):
        """
        Создать файл если его нет
        """
        if not exists(self.name_file):
            tmp_ = dirname(self.name_file)
            if not exists(tmp_):  # Если задан путь из папок если их нет
                makedirs(tmp_)  # Создаем путь из папок
                open(self.name_file, "w").close()
            else:
                # Если указано только имя файла без папок
                # или папки же существуют
                open(self.name_file, "w").close()

    def checkExistenceFile(self) -> bool:  # +
        """
        Проверить существование файла
        """
        return True if exists(self.name_file) else False

    def deleteFile(self):  # +
        """
        Удалить файл
        """
        # Удаление файла
        if self.checkExistenceFile():
            remove(self.route())

    def sizeFile(self) -> int:  # +
        """
        Получить размер файла
        """
        # Размер файла в байтах
        return getsize(self.name_file)

    def route(self) -> str:  # +
        """
        Получить абсолютный путь
        """
        # Путь к файлу
        return abspath(self.name_file)

    def createRoute(self):
        """
        Создать файл по указному пути, если нужно поместить в папки, то они создадутся
        """
        tmp_route: str = ""
        for folder_name in self.name_file.split('/')[:-1]:
            tmp_route += folder_name
            mkdir(tmp_route)
            tmp_route += '/'

    def removeRoute(self):
        """
        Удалить весь путь к файлу
        """
        rmtree(self.name_file.split('/')[1])

    def HashFileSha256(self) -> str:
        """
        Получить хеш сумму файла
        """
        return sha256sum(self.name_file)

    @abstractmethod
    def readFile(self, *arg) -> Any:
        ...

    @abstractmethod
    def writeFile(self, arg: Any):
        ...

    @abstractmethod
    def appendFile(self, arg: Any):
        ...


def ConcatData(callback: Callable, file_data: T_ConcatData, new_data: T_ConcatData):
    """
    Объединить два переменных, если они одинакового типа

    @param new_data: Текущие данные в `Python`
    @param file_data: Данные из файле
    @param callback: Вызовется при успешной проверки типов
    """

    if type(new_data) == type(new_data):
        match new_data:
            case list():
                file_data.extend(new_data)
            case tuple() | str():
                file_data += new_data
            case dict() | set():
                file_data.update(new_data)
            case _:
                raise TypeError("Не поддерживаемый тип")
        callback(file_data)
    else:
        raise TypeError("Тип данных в файле и тип входных данных различны")


def sha256sum(path_file: str):
    """
    Получить хеш сумму файла
    @param path_file: Путь к файлу
    """
    h = sha256()
    b = bytearray(128 * 1024)
    mv = memoryview(b)
    with open(path_file, 'rb', buffering=0) as f:
        for n in iter(lambda: f.readinto(mv), 0):
            h.update(mv[:n])
    return h.hexdigest()


def read_file_by_module(infile: str) -> ModuleType:
    """
    Импортировать файл как модуль `python`

    @param infile: Путь к `python` файлу
    @return: Модуль `python`
    """
    # указать модуль, который должен быть импортируется относительно пути модуль
    spec: Optional[ModuleSpec] = importlib.util.spec_from_file_location("my_module", infile)
    # создает новый модуль на основе спецификации
    __module: ModuleType = importlib.util.module_from_spec(spec)
    # выполняет модуль в своем собственном пространстве имен,
    # когда модуль импортируется или перезагружается.
    spec.loader.exec_module(__module)
    return __module


def concat_absolute_dir_path(_file: str, _path: str) -> str:
    """
    Получить абсолютный путь папки и объединить с другим путем

    :param _file:
    :param _path:
    :return:
    """
    return str(Path(_file).resolve().parent / _path)


def absolute_path_dir(_file: str, back: int = 1) -> Path:
    """
    Получить абсолютный путь к своей директории

    :param _file:
    """
    res = Path(_file).resolve()
    for _ in range(back):
        res = res.parent
    return res



from json import load, dump, JSONDecodeError
from typing import Any, Union

from .base_file import BaseFile, ConcatData


class JsonFile(BaseFile):
    """
    Работа с Json файлами
    """

    def __init__(self, name_file: str, type_file: str = ".json"):
        super().__init__(name_file, type_file=type_file)

    def readFile(self, **kwargs) -> Union[list, dict, int, str, float, None, bool]:
        try:
            with open(self.name_file, "r") as _jsonFile:
                return load(_jsonFile)
        except JSONDecodeError:
            return None

    def writeFile(self, data: Union[list, dict, int, str, float, None, bool, tuple],
                  *, indent=4,
                  skipkeys=False,
                  sort_keys=True,
                  ensure_ascii: bool = False):
        """
        :param data: list, dict, int, str, float, None, bool, tuple.
        :param skipkeys: Если False вызовет исключение при неправильном типе данных.
        :param indent: Отступы для записи.
        :param sort_keys: Сортировать ключи.
        :param ensure_ascii: Экранировать символы, если False данные запишутся как есть.
        """
        with open(self.name_file, "w") as _jsonFile:
            dump(data, _jsonFile, skipkeys=skipkeys, sort_keys=sort_keys, indent=indent, ensure_ascii=ensure_ascii)

    def appendFile(self, data: Union[list, dict[str, Any]], *, ensure_ascii: bool = False):
        ConcatData(
            lambda _data: self.writeFile(_data, ensure_ascii=ensure_ascii),
            self.readFile(),
            data)



from abc import ABCMeta
from typing import Any


class BaseSql(metaclass=ABCMeta):
    def __init__(self, user: str, password: str,
                 host: str = "localhost", ):
        self.SETTINGS_DB: dict[str, Any] | str = {"host": host,
                                                  "user": user,
                                                  "password": password}



import sys
# pip install typing-extensions
from typing import _type_check

from typing_extensions import _TypedDictMeta


class _DefTypeDictMeta(_TypedDictMeta):
    def __new__(cls, name, bases, ns, total=True):
        """Create new typed dict class object.

        This method is called when TypedDict is subclassed,
        or when TypedDict is instantiated. This way
        TypedDict supports all three syntax forms described in its docstring.
        Subclasses and instances of TypedDict return actual dictionaries.
        """
        for base in bases:
            if type(base) is not _DefTypeDictMeta:
                raise TypeError('cannot inherit from both a DefTypeDict type '
                                'and a non-TypedDict base class')
        tp_dict = type.__new__(_DefTypeDictMeta, name, (dict,), ns)

        annotations = {}
        own_annotations = ns.get('__annotations__', {})
        own_annotation_keys = set(own_annotations.keys())
        msg = "DefTypeDict('Name', {f0: t0, f1: t1, ...}); each t must be a type"
        own_annotations = {
            n: _type_check(tp, msg, module=tp_dict.__module__)
            for n, tp in own_annotations.items()
        }
        required_keys = set()
        optional_keys = set()

        for base in bases:
            annotations.update(base.__dict__.get('__annotations__', {}))
            required_keys.update(base.__dict__.get('__required_keys__', ()))
            optional_keys.update(base.__dict__.get('__optional_keys__', ()))

        annotations.update(own_annotations)
        if total:
            required_keys.update(own_annotation_keys)
        else:
            optional_keys.update(own_annotation_keys)

        tp_dict.__annotations__ = annotations
        tp_dict.__required_keys__ = frozenset(required_keys)
        tp_dict.__optional_keys__ = frozenset(optional_keys)
        if not hasattr(tp_dict, '__total__'):
            tp_dict.__total__ = total
        return tp_dict

    def default_dit(self, **kwargs):
        return {_k: _v for _k, _v
                in self.__dict__.items() if
                not _k.startswith("__") and
                not _k.endswith("__") and
                _k not in kwargs} | kwargs

    __call__ = default_dit  # dict  # static method


def DefTypeDict(typename, fields=None, /, *, total=True, **kwargs):
    """
    Словарь поддерживающий значения по умолчанию

    >>> class Settings(DefTypeDict):
    >>>    port: int = 7070
    >>>    host: str = "0.0.0.0"
    >>> Settings(port=8080)
    {"port":8080, host="0.0.0.0"}

    @param typename:
    @param fields:
    @param total:
    @param kwargs:
    @return:
    """

    if fields is None:
        fields = kwargs
    elif kwargs:
        raise TypeError("DefTypeDict takes either a dict or keyword arguments,"
                        " but not both")

    ns = {'__annotations__': dict(fields)}
    try:
        # Setting correct module is necessary to make typed dict classes pickleable.
        ns['__module__'] = sys._getframe(1).f_globals.get('__name__', '__main__')
    except (AttributeError, ValueError):
        pass

    return _DefTypeDictMeta(typename, (), ns, total=total)


_DefTypeDict = type.__new__(_DefTypeDictMeta, 'DefTypeDict', (), {})
DefTypeDict.__mro_entries__ = lambda bases: (_DefTypeDict,)



import sqlite3
from os import remove
from os.path import exists, abspath
from re import findall
from sqlite3 import Binary
from typing import List, Tuple, Dict, Union

from mg_file.file.txt_file import TxtFile
from mg_file.sqllite_orm.sqlmodules import *


class SqlLiteQrm:
    __slots__ = ("__name_db", "__header_table")

    def __init__(self, name_dbf: str) -> None:  # +

        tmp = name_dbf.split(".")  # Проверка того что разшерение db
        if len(tmp) != 2 or tmp[1] != "db":
            raise NameError("Файл должен иметь разшерение .db")

        self.__name_db = name_dbf
        self.__header_table: Dict[
            str, Dict[str, tuple]] = self.__update_header_table()  # Тут храниться типы столбцов таблциы

    @property
    def name_db(self):
        return self.__name_db

    @name_db.setter
    def name_db(self, next_name_db: str):
        self.__name_db = next_name_db
        self.__header_table: Dict[
            str, Dict[str, tuple]] = self.__update_header_table()  # Тут храниться типы столбцов таблциы

    @property
    def header_table(self):
        return self.__header_table

    @staticmethod
    def __CheckBlob(data: List[Union[List[Union[str, bytes, Binary, int, float]], Tuple]]) -> \
            List[Union[List[Union[str, bytes, Binary, int, float]], Tuple]]:
        if type(data[0]) == tuple:
            raise TypeError("Нельзя использовать параметер CheckBLOB на структуре данных tuple. используйте list")
        return [[Binary(item2) if type(item2) == bytes else item2 for item2 in item] for item in data]

    @staticmethod
    def __print_table(NameTable: str,
                      head: str,
                      data_list: List[Union[list, tuple]],
                      width_table: int = 5,
                      ) -> str:
        res: str = ""
        head = head.split(", ")
        for i, p in enumerate(head):
            head[i] = p.center(width_table)
        head = '¦'.join(head)
        lain = "+{0}+".format("-" * len(head))

        res += lain + "\n"
        res += "|{}|\n".format(NameTable.center(len(lain) - 2))
        res += lain + "\n"
        res += "|{0}|\n".format(head)

        for p in data_list:
            res += lain + "\n"
            p = list(p)
            for i, d in enumerate(p):
                p[i] = str(d).center(width_table)

            res += "|{}|\n".format('¦'.join(p))
        res += lain + "\n"

        return res

    def __update_header_table(self) -> Dict[str, Dict[str, tuple]]:  # +
        # Получение схемы всех таблиц в бд
        res: Dict[str, Dict[str, tuple]] = {}

        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            for x in self.ListTables():
                meta = cursor.execute("PRAGMA table_info({0})".format(x))

                """
                (номер имни, имя, тип данных,NOT NULL, значение по умолчанию, PRIMARY KEY )
                """
                res[x] = {}
                for r in meta:

                    type_name = r[2]

                    res_type = None
                    if type_name == "TEXT":
                        res_type = str
                    elif type_name == "INTEGER":
                        res_type = int
                    elif type_name == "REAL":
                        res_type = float
                    elif type_name == "BLOB":
                        res_type = bytes

                    res_nn: str = ""
                    if r[3] == 1:
                        res_nn = " NOT NULL"

                    if r[4]:  # != None:
                        res_nn += " DEFAULT {0}".format(r[4])

                    if r[5] == 1:
                        res_nn = " PRIMARY KEY"

                    res[x].update({r[1]: f"{toTypeSql(res_type)}{res_nn}"})

        return res

    # Информация о БД
    def ListTables(self) -> List[str]:  # +
        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
            return [x[0] for x in cursor.fetchall() if x[0] != 'sqlite_sequence']

    def HeadTable(self, NameTable, width_table: int = 10) -> str:
        res = self.__print_table(NameTable, ", ".join(self.header_table[NameTable].keys()),
                                 [[str(x) for x in self.header_table[NameTable].values()]], width_table)
        return res

    def GetTable(self, NameTable: str,
                 LIMIT: Tuple[int, int] = None,
                 FlagPrint: int = 0
                 ) -> list:  # +
        """
        Конвертация bytes в обьекта SQl BLOB и обратно
        a = Binary(b"101101")
        print(a.tobytes())
        """

        request: str = 'SELECT * FROM {0}'.format(NameTable)

        if LIMIT:
            request += " LIMIT {0} OFFSET {1}".format(LIMIT[0], LIMIT[1])

        # Получить данные из таблицы
        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            cursor.execute(request)
            res = cursor.fetchall()
            if FlagPrint:
                print(self.__print_table(NameTable, ", ".join(self.header_table[NameTable].keys()), res, FlagPrint))
            return res

    def GetColumn(self, NameTable: str, name_columns: str, LIMIT: Tuple[int, int] = None) -> list:  # +
        # Получить данные из столбца

        request = 'SELECT {0} FROM {1}'.format(name_columns, NameTable)
        if LIMIT:
            request += " LIMIT {0} OFFSET {1}".format(LIMIT[0], LIMIT[1])

        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            cursor.execute(request)
            return [x[0] for x in cursor.fetchall()]

    # Удаление Данных
    def DeleteTable(self, NameTable: Union[str, List[str]]):  # +
        # Удалить несколько таблиц
        if type(NameTable) == list:
            for item in NameTable:
                if self.header_table.get(item):
                    self.header_table.pop(item)
                with sqlite3.connect(self.__name_db) as connect:
                    connect.cursor().execute(f"DROP TABLE IF EXISTS {item}")  # Удалить таблицу если она существует
        else:
            if self.header_table.get(NameTable):
                self.header_table.pop(NameTable)
            with sqlite3.connect(self.__name_db) as connect:
                connect.cursor().execute(f"DROP TABLE IF EXISTS {NameTable}")  # Удалить таблицу если она существует

    def DeleteLineTable(self,
                        NameTable: str,
                        sqlWHERE: str = ""):
        """
        :param NameTable: Название таблицы
        :param sqlWHERE: Условие SQL после WHERE
        """
        request: str = "DELETE FROM {0}".format(NameTable)

        if sqlWHERE:
            request += " WHERE {0}".format(sqlWHERE)

        with sqlite3.connect(self.__name_db) as connect:
            connect.cursor().execute(request)

    # Работа с данными таблиц
    def CreateTable(self, NameTable: str, columns: Union[str, Dict]):  # +
        # Конвертация типов в str
        request: str = f"CREATE TABLE IF NOT EXISTS {NameTable} "
        if type(columns) == str:
            request += columns

            dict_tmp: dict = {}
            for items in (x.strip() for x in columns.replace("(", "").replace(")", "").split(",")):
                arr = items.split(' ')
                dict_tmp[arr[0]] = " ".join(arr[1::])

            self.header_table[NameTable] = dict_tmp

        elif type(columns) == dict:
            # Если все данные сделаны как положено
            if list(filter(lambda x: True if type(x) == str else False, columns.values())):
                request += "( "
                for k, v in columns.items():
                    request += f"{k} {v}, "
                request = request[:-2:] + " )"
                self.header_table[NameTable] = columns
            else:
                raise TypeError("Переданные данные не типа str")

        # Создание таблицы
        with sqlite3.connect(self.__name_db) as connect:
            connect.cursor().execute(request)

    def ExecuteTable(self, NameTable: str,
                     data: Union[str, int, float,
                                 List[Union[str, bytes, int, float]],
                                 Tuple,
                                 Dict[str, Union[str, bytes, int, float]]] = None,
                     sqlRequest: str = "", *,
                     CheckBLOB: bool = False
                     ):  # +
        """
        :param NameTable:
        :param data:
        :param sqlRequest:
        :param CheckBLOB: проверка структуры на наличие бинарных бинарных данных и перевод их  в sqlite3.Binary()
        :return:
        """

        # Создать тест на проверку одной записи
        request: str = "INSERT INTO {0}".format(NameTable)

        if sqlRequest:  # для вложенных запасов
            request += " {0}".format(sqlRequest)
            data = None  # Сделать тест проверки вставки данных

        else:
            if type(data) in (int, float, str):  # Для SQL команд
                if type(data) == str and data.find("bytes") != -1:
                    raise TypeError(
                        "Нельзя отпаять BLOB в формате строки. Воспользуйтесь добавление данных через list")
                request += " ('{0}') VALUES ({1})".format("', '".join(self.header_table[NameTable].keys()), data)
                data = None

            else:  # Для структур данных
                res: str = ', '.join('?' * len(data))
                # Конвертация типа в dict в SQL запрос
                if type(data) == dict:
                    if tuple(data.keys() - self.header_table[NameTable].keys()):
                        raise IndexError("Имён переданного столбца не существует")

                    request += " ('{0}') VALUES ({1})".format("', '".join(data.keys()), res)

                    if CheckBLOB:
                        data = list(data.values())
                        for index1, tup in enumerate(data):
                            if type(tup) == bytes:
                                data[index1] = Binary(tup)
                        data = tuple(data)
                    else:
                        data = tuple(data.values())

                # Конвертация типов list, tuple в SQL запрос
                elif type(data) == tuple or type(data) == list:
                    if len(data) != len(self.header_table[NameTable]):
                        raise IndexError("Разное количество столбцов таблицы и входных данных")

                    request += " ('{0}') VALUES ({1})".format("', '".join(self.header_table[NameTable].keys()), res)

        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            cursor.execute(request, data) if data else cursor.execute(request)

    def ExecuteManyTable(self, NameTable: str,
                         data: List[Union[List[Union[str, bytes, Binary, int, float]], Tuple]],
                         head_data: Union[List[str], Tuple] = None, *,
                         CheckBLOB: bool = False
                         ):  # +
        """
        :param NameTable:
        :param data:
        :param CheckBLOB: проверка стурктуры на анличие бинарных бинырных данных и перевод их  в sqlite3.Binary()
        :param head_data: собственные заголовки
        :return:
        """
        if type(data) != list:
            raise TypeError("Должен быть тип List")

        # Для получения имен параметров name_head_data
        if not head_data:
            head_data = tuple(self.header_table[NameTable].keys())  # "', '".join(self.header_table[name_table].keys())

        # Нахождение в массиве данных типа bytes и перевод их через sqlite3.Binary()
        if CheckBLOB:
            data = self.__CheckBlob(data)

        res: str = ', '.join('?' * (len(head_data)))
        request: str = "INSERT INTO {nt} ('{name_arg}') VALUES ({values})".format(nt=NameTable,
                                                                                  name_arg="', '".join(head_data),
                                                                                  values=res)
        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            cursor.executemany(request, data)

    def ExecuteManyTableDict(self, NameTable: str, data: List[Dict]):  # +
        for dict_x in data:
            res: str = ', '.join('?' * len(dict_x))
            ae = "INSERT INTO {nt} {name_arg} VALUES ({values})".format(nt=NameTable,
                                                                        name_arg=tuple(dict_x.keys()),
                                                                        values=res)
            with sqlite3.connect(self.__name_db) as connect:
                cursor = connect.cursor()
                cursor.execute(ae, tuple(dict_x.values()))

    def UpdateColumne(self, NameTable: str,
                      name_column: Union[str, List[str]],
                      new_data: Union[str, bytes, int, float, List[Union[str, bytes, int, float]]],
                      sqlWHERE: str = ""
                      ):
        """
        Обновление данных в столбцах
        :param NameTable: Название таблицы
        :param name_column: Название столбца который будет выбора
        :param new_data: Новое значение у столбцов
        :param sqlWHERE: Условие SQL поле WHERE
        """

        request: str = "UPDATE {0} SET ".format(NameTable)

        if type(name_column) == list and type(new_data) == list:  # Несколько столбцов на изменение
            if len(name_column) != len(new_data):
                raise IndexError("name_column != new_data")

            for n, d in zip(name_column, new_data):
                if type(n) == str:
                    request += "{0} = '{1}', ".format(n, d)
                else:
                    request += "{0} = {1}, ".format(n, d)
            request = request[:-2:]
        else:  # один столбец на измнение
            request += "{0} = {1}".format(name_column, new_data)

        if sqlWHERE:
            request += " WHERE {0}".format(sqlWHERE)

        with sqlite3.connect(self.__name_db) as connect:
            connect.cursor().execute(request)

    # Поиск в таблице
    def Search(self,
               objectSelect: Select,
               *,
               FlagPrint: int = 0,
               ) -> Union[str, list]:

        request = objectSelect.Request

        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            cursor.execute(request)
            res = cursor.fetchall()

        if FlagPrint:
            sqlSelect = "".join(findall(r"SELECT ([\w, *()'\\]+) FROM",
                                        request))  # "".join(findall(r"SELECT ([\w, *]+) FROM", request))
            NameTable = "".join(findall(r"SELECT [\w, *()'\\]+ FROM ([\w]+)[A-Z]*",
                                        request))  # "".join(findall(r"SELECT [\w, *]+ FROM ([\w]+)[A-Z]*", request))
            if sqlSelect == "*":
                sqlSelect = ", ".join(self.header_table[NameTable].keys())
            print(self.__print_table(NameTable, sqlSelect, res, FlagPrint))

        return res

    # Операции над БД
    def SaveDbToFile(self, name_save_db):
        tm = TxtFile(name_save_db)
        tm.deleteFile()
        with sqlite3.connect(self.__name_db) as connect:
            for sql_request in connect.iterdump():
                tm.appendFile(sql_request)

    def ReadFileToDb(self, name_read_db):
        tm = TxtFile(name_read_db)
        with sqlite3.connect(self.__name_db) as connect:
            cursor = connect.cursor()
            # Это все для того чтобы пропустить эти строки которые возникают непонятно почему
            # DELETE FROM "sqlite_sequence";INSERT INTO "sqlite_sequence" VALUES('stocks',3);
            cursor.execute("BEGIN TRANSACTION;")
            tmp = False
            for text in tm.readFile().split(";"):
                if tmp:
                    cursor.execute(text)
                else:
                    if text.find("CREATE") != -1:
                        tmp = True
                        cursor.execute(text)

    def DeleteDb(self):  # +
        if exists(self.__name_db):
            remove(abspath(self.__name_db))


if __name__ == '__main__':
    pass



from csv import reader, writer
from os import SEEK_END, SEEK_SET
from typing import Union, List

# pip install prettytable
from prettytable import PrettyTable

from .base_file import BaseFile, ConcatData


class CsvFile(BaseFile):
    def __init__(self, name_file: str, type_file: str = ".csv"):
        super().__init__(name_file, type_file=type_file)

    def readFile(self,
                 *,
                 encoding: str = "utf-8",
                 limit: int = 0,
                 miss_get_head=False,
                 delimiter=",",
                 ) -> list[list[str]]:
        """
        :param limit: Лимит чтения записей
        :param miss_get_head: Пропустить чтение заголовка
        :param delimiter: Символ, который будет разделять колонки
        :param encoding: Кодировка
        """
        _res = []
        with open(self.name_file, "r", encoding=encoding) as _csvFile:
            _reader = reader(_csvFile, delimiter=delimiter)

            if limit:  # Если есть лимит для чтения записей
                for _index, _row in enumerate(_reader):
                    if _index < limit:
                        _res.append(_row)
                    else:
                        break
            else:  # Если лимита нет, то читаем все записи
                _res = list(_reader)

            if miss_get_head:  # Если нужно пропустить заголовки
                return _res[1::]

            else:
                return _res

    def readFileAndFindDifferences(self, new_data_find: List[List], funIter) -> bool:  # +
        """
        for new_data, data_file in zip(self.ListStock, DataFile):
            if new_data != data_file:
                funIter(new_data)

        :param new_data_find: Новые данные
        :param funIter: Функция которая будет выполняться на каждой итерации
        """
        data_file = self.readFile(miss_get_head=True)
        if data_file != new_data_find:
            for _ in (funIter(new_data) for new_data in new_data_find if new_data not in data_file):
                continue
            return True
        else:
            return False

    def readFileRevers(self, *,
                       limit: int = None,
                       encoding: str = "utf-8",
                       newline: str = ""
                       ) -> List[List[str]]:
        def reversed_lines(file):
            # Generate the lines of file in reverse order
            part = ''
            for block in reversed_blocks(file):
                for c in reversed(block):
                    if c == '\n' and part:
                        yield part[::-1]
                        part = ''
                    part += c
            if part:
                yield part[::-1]

        def reversed_blocks(file, block_size=4096):
            # Generate blocks of file's contents in reverse order.
            file.seek(0, SEEK_END)
            here = file.tell()
            while 0 < here:
                delta = min(block_size, here)
                here -= delta
                file.seek(here, SEEK_SET)
                yield file.read(delta)

        res = []
        with open(self.name_file, "r", encoding=encoding, newline=newline) as f:

            if limit:  # Лимит чтения строк
                for row in reader(reversed_lines(f)):
                    if limit:
                        res.append(row)
                        limit -= 1
                    else:
                        break
            else:
                for row in reader(reversed_lines(f)):
                    res.append(row)
        return res

    def writeFile(self,
                  data: Union[list[Union[str, int, float]],
                              list[list[Union[str, int, float]]]],
                  *,
                  header: tuple = None,
                  FlagDataConferToStr: bool = False,
                  encoding: str = "utf-8",
                  delimiter=",",
                  newline="",
                  ):
        """
        @param data:
        @param header: Эти данные будут заголовками
        @param FlagDataConferToStr: Переводит все данные в формат str
        @param delimiter: Символ, который будет разделять колонки
        @param encoding: Кодировка
        @param newline:
        """
        with open(self.name_file, "w", encoding=encoding, newline=newline) as _csvFile:
            _writer = writer(_csvFile, delimiter=delimiter)

            if header:  # Запись заголовка
                _writer.writerow(header)

            if FlagDataConferToStr:
                if type(data[0]) != list:
                    data = [str(n) for n in data]
                else:
                    data = [[str(n) for n in m] for m in data
                            # Проверить что объект можно перебрать
                            if getattr(m, "__iter__", False)]

            if type(data[0]) != list:
                _writer.writerow(data)
            else:
                _writer.writerows(data)

    def appendFile(self, data: Union[list[Union[str, int, float]],
                                     list[list[Union[str, int, float]]]],
                   *,
                   FlagDataConferToStr: bool = False,
                   encoding: str = "utf-8",
                   delimiter=",",
                   ):
        """
        :param data:
        :param FlagDataConferToStr: Переводит все данные в формат str
        :param delimiter: Символ, который будет разделять колонки
        :param encoding: Кодировка
        """
        ConcatData(
            lambda _data: self.writeFile(_data,
                                         FlagDataConferToStr=FlagDataConferToStr,
                                         encoding=encoding,
                                         delimiter=delimiter),
            self.readFile(),
            data)

    @staticmethod
    def ptabel(data: list, align="l"):
        """
        :param data:
        :param align:
        :return:
        """
        x = PrettyTable(data[0])
        x.add_rows(data[1:])
        x.align = align
        return x



"""

"""

from .async_base_sql import AsyncBaseCommand

try:
    # https://magicstack.github.io/asyncpg/current/installation.html
    # pip install asyncpg
    from asyncpg import create_pool
    from psycopg2 import OperationalError
except ModuleNotFoundError:
    pass


class Config(AsyncBaseCommand):

    def __init__(self, user: str,
                 password: str,
                 database: str | None = None, port: int = 5432,
                 host: str = "localhost"):
        super().__init__(user, password, host)
        self.SETTINGS_DB.update({"port": port, })
        if database:
            self.SETTINGS_DB["database"] = database

    # self.SETTINGS_DB = ' '.join([f"{_k}={_v}" for _k, _v in self.SETTINGS_DB.items()])

    async def read_command(self, _connection,
                           execute: str,
                           params: tuple | dict | list = (),
                           ) -> list[object]:
        return await _connection.fetch(execute, *params)

    async def transaction_read_command(self, _connection,
                                       execute: str,
                                       params: tuple | dict | list = (),
                                       ) -> list[object]:
        async with _connection.transaction():
            return await _connection.fetch(execute, *params)

    async def mutable_command(self, _connection,
                              execute: str,
                              params: tuple | dict | list = (), ):
        await _connection.execute(execute)

    async def transaction_mutable_command(self, _connection,
                                          execute: str,
                                          params: tuple | dict | list = ()):
        async with _connection.transaction():
            return await _connection.execute(execute)

    async def test(self, execute) -> list[object]:
        """
        https://magicstack.github.io/asyncpg/current/api/index.html

        - fetch(query, *args, timeout=None) → list[source]
          Запустите запрос и верните результаты в виде списка Record.

        - fetchrow(query, *args, timeout=None)[source]
           Запустите запрос и верните первую строку.

        - fetchval(query, *args, column=0, timeout=None)[source]
            Запустите запрос и верните значение в первой строке.

        - execute(query: str, *args, timeout: Optional[float] = None) → str[source]
            Выполните команду (или команды) SQL.
        """

        async with self.ConnectCallback(**self.SETTINGS_DB) as pool:
            # Take a connection from the pool.
            async with pool.acquire() as connection:
                # Open a transaction.
                # async with connection.transaction():
                # Run the query passing the request argument.
                # result = await connection.execute(execute)
                return await connection.fetch(execute)

    @property
    def ConnectCallback(self):
        return create_pool

    @property
    def ERROR(self):
        return OperationalError



from .base_file import BaseFile, ConcatData, sha256sum
from .base_folder import BaseFolder, Folder, DiffDir
from .csv_file import CsvFile
from .env_file import EnvFile
from .json_file import JsonFile
from .pickle_file import PickleFile
from .txt_file import TxtFile
from .yaml_file import YamlFile



NAME_DB = "fast_test"
NAME_TABEL = "test_tabel"
HOST = "51.250.27.5"
Refresh_TABLE = f"""
DROP TABLE IF EXISTS {NAME_TABEL};
CREATE TABLE {NAME_TABEL}
(
    id    serial PRIMARY KEY,
    email varchar(255),
    buy   money
);
"""



import unittest

from mg_file import PickleFile


class TestPickleFile(unittest.TestCase):
    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.name_file = "test_pickle.pkl"

    def setUp(self):
        self.pk = PickleFile(self.name_file)
        self.pk.deleteFile()

    def test_writeFile_and_readFile(self):
        # Проверка записи данных
        test_data = [
            (1, 2, 3, 4),
            [12, 23, 221],
            ["1231", 12, (2, 22)],
            {213123, 123213},
            {'s1': '213'},
        ]

        for td in test_data:
            self.pk.writeFile(td)
            self.assertEqual(self.pk.readFile(), td)
            self.pk.deleteFile()

        self.pk.deleteFile()

    def test_appendFile(self):
        test_data = [1, 2, 3, 4]
        new_data = [98, 678, 88]
        self.pk.writeFile(test_data)
        self.pk.appendFile(new_data)
        test_data += new_data
        self.assertEqual(self.pk.readFile(), test_data)



"""
Модификаторы данных
"""


def definition(TypeColumn):
    if TypeColumn == str:
        res = "TEXT"
    elif TypeColumn == int:
        res = "INTEGER"
    elif TypeColumn == float:
        res = "REAL"
    elif TypeColumn == bytes:
        res = "BLOB"
    elif TypeColumn is None:
        res = "NULL"
    else:
        raise TypeError(f"Указан не верный тип данных выбирете str;int;float;None;bytes\n{TypeColumn}")
    return res


# Должны содержать уникальные значения
PrimaryKey = lambda TypeColumn: definition(TypeColumn) + " PRIMARY KEY"
# Всегда должно быть заполнено
NotNull = lambda TypeColumn: definition(TypeColumn) + " NOT NULL"
# Все столбцы будут по умолчанию заполнены указанными значениями
NotNullDefault = lambda TypeColumn, default: definition(TypeColumn) + f" NOT NULL DEFAULT {default}"
# Значение по умолчанию
Default = lambda TypeColumn, default: definition(TypeColumn) + " DEFAULT {0}".format(default)
# Авто заполнение строки. подходит для id
PrimaryKeyAutoincrement = lambda TypeColumn: definition(TypeColumn) + " PRIMARY KEY AUTOINCREMENT"
toTypeSql = lambda TypeColumn: definition(TypeColumn)

"""
Агрегирующие функции
"""
CountSql = lambda arg: "count(DISTINCT {0})".format(arg[1::]) if arg[0] == "-" else "count({0})".format(arg)
SumSql = lambda arg: "sum(DISTINCT {0})".format(arg[1::]) if arg[0] == "-" else "sum({0})".format(arg)
AvgSql = lambda arg: "avg(DISTINCT {0})".format(arg[1::]) if arg[0] == "-" else "avg({0})".format(arg)
MinSql = lambda arg: "min(DISTINCT {0})".format(arg[1::]) if arg[0] == "-" else "min({0})".format(arg)
MaxSql = lambda arg: "max(DISTINCT {0})".format(arg[1::]) if arg[0] == "-" else "max({0})".format(arg)


class Select:

    def __init__(self, name_table, *select_arg, req=None):

        self.Request: str = ""
        if req:
            self.Request += req

        if name_table:
            if name_table == "*":
                raise ValueError(f"{name_table} не может иметь название *")
            self.Request = 'SELECT {0} FROM {1}'.format(', '.join(select_arg), name_table)

    def Join(self, name_table: str, ON: str, leftJoin: bool = False):
        if not leftJoin:
            self.Request += " INNER JOIN {0} ON {1}".format(name_table, ', '.join(ON)) \
                if type(ON) == tuple \
                else " INNER JOIN {0} ON {1}".format(name_table, ON)
        else:
            self.Request += " LEFT JOIN {0} ON {1}".format(name_table, ', '.join(ON))
        return Select("", "", req=self.Request)

    def GroupBy(self, *name_column):
        self.Request += " GROUP BY {0}".format(', '.join(name_column)) if list(
            filter(lambda it: True if it else False, name_column)) else ""
        return Select("", "", req=self.Request)

    def OrderBy(self, NameColumn: str):
        self.Request += f" ORDER BY {NameColumn[1::]} DESC" if NameColumn[0] == '-' else f' ORDER BY {NameColumn} ASC'
        return Select("", "", req=self.Request)

    def Where(self, sqlWhere: str):
        self.Request += f" WHERE {sqlWhere}"
        return Select("", "", req=self.Request)

    def Limit(self, end: int, offset: int = 0):
        self.Request += f" LIMIT {end} OFFSET {offset}" if end else ""
        return Select("", "", req=self.Request)



from abc import abstractmethod
from pprint import pformat
from typing import Any, Callable

from loguru import logger

from .sync_serializer import Efetch
from ..base_sql import BaseSql


class SyncBaseSql(BaseSql):

    def connect_db(self, fun: Callable, *args, **kwargs) -> Any:
        """
        Так как у нас может быть подключение с транзакцией и без, мы не можем использовать
        контекстный менеджер `with`, так как он обязательно создает транзакцию.
        Поэтому мы сами обрабатываем исключения и закрываем соединение с БД.
        """
        connection = self.CONNECT(**self.SETTINGS_DB)
        try:
            return fun(connection, *args, **kwargs)
        except self.ERROR as e:
            logger.error(e)
            raise e
        finally:
            connection.close()

    def rsql(self, execute: str, params: tuple | dict | list = (), tdata: Callable = Efetch.all,
             ) -> tuple[str, tuple | dict | list]:
        """
        Чтение из БД
        """
        return self.connect_db(self.read_command, execute=execute, params=params, tdata=tdata)

    def Rsql(self, execute: str, params: tuple | dict | list = (), tdata: Callable = Efetch.all) -> str:
        """
        Чтение из БД с красивым выводом в консоль
        """
        return self.pprint_deco(self.rsql(execute, params, tdata))

    def wsql(self, execute: str,
             params: tuple | dict | list = (),
             autocommit: bool = False) -> tuple[str, tuple | dict | list]:
        """
        Внесение изменений в БД

        @param execute: `SQL` команда
        @param params: Вставить данные в `SQL` команду
        @param autocommit: Если вы используете `autocommit` то передувайте одну `SQL` команду. Установите `True`
        если вы выполняете команды связанные с СУБД.
        """
        return self.connect_db(self.mutable_command, execute=execute, params=params, autocommit=autocommit)

    @staticmethod
    def pprint_deco(d: Any) -> str:
        """
        Декоратор для красивого вывода результата функции в консоль
        """
        return pformat(d)

    @abstractmethod
    def read_command(self, _connection,
                     execute: str,
                     params: tuple | dict | list = (),
                     tdata: Callable = Efetch.all,
                     *args,
                     **kwargs):
        """
        Декоратор для выполнения чтения из БД
        """
        return NotImplemented()

    @abstractmethod
    def mutable_command(self, _connection,
                        execute: str,
                        params: tuple | dict | list = (),
                        *args,
                        **kwargs):
        """
        Декоратор для выполнения изменяемой SQL команды
        """
        return NotImplemented()

    @property
    @abstractmethod
    def CONNECT(self):
        pass

    @property
    @abstractmethod
    def ERROR(self):
        ...



"""
"""

from typing import Any, Callable

from .async_base_sql import AsyncBaseCommand
from .async_serializer import Efetch

try:
    # https://aiopg.readthedocs.io/en/stable/core.html
    # pip install aiopg
    from aiopg import connect
    from psycopg2 import OperationalError
except ModuleNotFoundError:
    pass


class Config(AsyncBaseCommand):

    def __init__(self,
                 user: str,
                 password: str,
                 database: str | None = None,
                 port: int = 5432,
                 host: str = "localhost"):
        super().__init__(user, password, host)
        self.SETTINGS_DB.update({
            "port": port,
        })
        if database:
            self.SETTINGS_DB["dbname"] = database
        # self.SETTINGS_DB = ' '.join([f"{_k}={_v}" for _k, _v in self.SETTINGS_DB.items()])

    async def read_command(self, _connection,
                           execute: str,
                           params: tuple | dict | list = (),
                           tdata: Callable = Efetch.all, ) -> Any:
        async with _connection.cursor() as _cur:
            await _cur.execute(execute, params)
            return await tdata(_cur)

    async def transaction_read_command(self, _connection,
                                       execute: str,
                                       params: tuple | dict | list = ()) -> Any:
        pass

    async def mutable_command(self, _connection,
                              execute: str,
                              params: tuple | dict | list = (),
                              ):
        async with _connection.cursor() as _cur:
            await _cur.execute(execute, params)
        return _cur.statusmessage

    async def transaction_mutable_command(self, _connection,
                                          execute: str,
                                          params: tuple | dict | list = ()):
        pass

    @property
    def ERROR(self):
        return OperationalError

    @property
    def ConnectCallback(self):
        return connect



from abc import ABCMeta
from collections import deque
from collections import namedtuple
from enum import Enum
from typing import Any, Coroutine, Callable


class BaseTasks(metaclass=ABCMeta):
    """
    Базовый класс для списка асинхронных задач.
    Используются как примесь к асинхронному классу `AsyncBaseSql`
    """

    def __init__(self):
        self.tasks: deque[Coroutine] = deque()

    def appendTask(self, coroutine: Coroutine):
        """Добавить здание в список"""
        self.tasks.append(coroutine)

    def extendTask(self, coroutine: list[Coroutine] | deque[Coroutine]):
        """Расширить список задач другим списком список"""
        self.tasks.extend(coroutine)


async def dictfetchall(cursor) -> list[dict[str, Any]]:
    """
    Вернуть в виде словаря
    """
    columns = [col[0] for col in cursor.description]
    return [
        dict(zip(columns, row))
        for row in await cursor.fetchall()
    ]


async def namedtuplefetchall(cursor) -> list[namedtuple]:
    """
    Вернуть в виде именовано го картежа
    """
    desc = cursor.description
    nt_result = namedtuple('_', [col[0] for col in desc])
    return [nt_result(*row) for row in await cursor.fetchall()]


async def _fetchone(cursor):
    return await cursor.fetchone()


async def _fetchall(cursor):
    return await cursor.fetchall()


class Efetch(Enum):
    dict_: Callable[[Any], list[dict[str, Any]]] = dictfetchall
    namedtuple: Callable[[Any], list[namedtuple]] = namedtuplefetchall
    one: Callable[[Any], Any] = _fetchone
    all: Callable[[Any], Any] = _fetchall



from abc import ABCMeta
from collections import deque
from os import sep, listdir, path
from re import match
from typing import NamedTuple, Union

from .base_file import sha256sum


class DiffDir(NamedTuple):
    """
    Объект для хранения разницы меду двумя директориями
    """
    # Текущая директория (А)
    infloder: str
    # Сравнительная директория (Б)
    outfolder: str
    # Файлы, которые не существую в директории (Б)
    not_exist_arr_file: set[str]
    # Папки, которые не существуют в директории (Б)
    not_exist_arr_folder: set[str]
    # Данные в (Б) отличаются от данных в (А)
    diff_data_arr_file: set[str]
    # Файлы, которые нарушают исключение и существуют в (Б)
    file_intruder: set[str]
    # Папки, которые нарушают исключение и существуют в (Б)
    folder_intruder: set[str]

    # def log(self):
    #     logger.info(f"NotFile:\n{pformat(self.not_exist_arr_file)}")
    #     logger.info(f"NotFolder:\n{pformat(self.not_exist_arr_folder)}")
    #     logger.info(f"DiffHashFile:\n{pformat(self.diff_data_arr_file)}")
    #     logger.info(f"FileIntruder:\n{pformat(self.file_intruder)}")
    #     logger.info(f"FolderIntruder:\n{pformat(self.folder_intruder)}")


class BaseFolder(metaclass=ABCMeta):
    # Разделитель пути, будет актуальным для каждой ОС
    OsSeparator: str = sep

    def __init__(self, folder: str):
        # Директория из которой нужно получить файлы и папки
        self.folder: str = folder

    def getAllFileAndFolder(self) -> dict[str, Union[set[str], str]]:
        """
        Получить список всех файлов и папок по указному пути.

        @return: Список файлов и список папок
        """

        # Список со всеми файлами
        arr_file: set[str] = set()
        # Список со всеми папками
        arr_folder: set[str] = set()
        ###################################
        # Изначальный путь к папке
        _path: str = self.folder
        # Временный список с папками для перебора
        _arr_select_folder: deque = deque([self.folder])
        _Live = True
        while _Live:
            # Выбираем первый элемент путь из очереди
            _path = _arr_select_folder[0]
            # Перебираем все файлы и папки в пути
            for x in listdir(_path):
                # Создаем полный путь
                file = path.join(_path, x)
                # Если папка
                if path.isdir(file):
                    # Добавляем в результирующий массив
                    # Если имя переименовали то создаем объект `TypeReplaceName
                    arr_folder.add(file.replace(self.folder, ''))
                    # Добавляем в список перебора путей
                    _arr_select_folder.append(file)
                # Если файл
                else:
                    # Добавляем в результирующий список файлов
                    # Если имя переименовали то создаем объект `TypeReplaceName
                    arr_file.add(file.replace(self.folder, ''))
            # Удаляем путь, который перебрали
            _arr_select_folder.popleft()
            # Если путей нет, то прекращаем перебор
            if len(_arr_select_folder) == 0:
                _Live = False

        return {
            "split_path": self.folder,
            "arr_file": arr_file,
            "arr_folder": arr_folder,
        }

    @classmethod
    def sortPath(cls, arr_path: Union[list[str], set[str]], reverse: bool = True) -> list[str]:
        """
        Отсортировать пути.
        Для создания и удаления папок необходимо соблюдать порядок путей.

        @param arr_path: Список путей
        @param reverse: Сортировать в обратном порядке.
         Сначала длинный путь, который будет указывать на файл, в конце, короткий путь,
         который будет указывать на папку. Когда мы будем удалять файлы и папки то нам
         нужно отсортировать пути в обратном порядке. Когда нам нужно создать файлы и папки то
         нам нужен обычный порядок.
        @return: Отсортированные пути
        """
        # Создаем список с количеством разделителей директорий
        _res: list[tuple[int, str]] = [(len(_x.split(cls.OsSeparator)), _x) for _x in list(arr_path)]
        # Сортируем директории по количеству разделителей, в обратном порядке
        _res.sort(key=lambda k: k[0], reverse=reverse)
        # Преобразуем данные
        _res: list[str] = [_x[1] for _x in _res]
        return _res

    @staticmethod
    def excludeFolderAndFile(arr_exclude: set[str], *,
                             arr_file: set[str],
                             arr_folder: set[str], **kwargs) -> tuple[set[str], set[str]]:
        """
        Метод для исключения фалов и папок из списков.

        @param arr_exclude: Список исключений
        @param arr_file: Список файлов
        @param arr_folder: Список папок
        @return: Новый список файлов и папок, с исключенными путями
        """

        # Если нечего исключать, то возвращаем исходные данные
        if not arr_exclude:
            return arr_file, arr_folder

        def isExclude(_path: str) -> bool:
            # Проверяем путь на вхождение в список исключений
            for _re in arr_exclude:
                # Проверяем начло не соответствие шаблону исключения
                isExist = match(fr"{_re}[\W\w]*", _path.__str__())
                if isExist:
                    return True
            return False

        def logic(_arr_path) -> set[str]:
            _right_path: set[str] = set()
            # Перебираем пути
            for _path in _arr_path:
                # Если путь НЕ нужно исключить, то добавляем его в правильный путь
                if not isExclude(_path):
                    _right_path.add(_path)
            return _right_path

        return logic(arr_file), logic(arr_folder)


class Folder(BaseFolder):
    @staticmethod
    def _dirDiff(
            in_folder: str,
            arr_file_in: set[str], arr_folder_in: set[str],
            outfolder: str,
            arr_file_out: set[str], arr_folder_out: set[str],
    ) -> DiffDir:
        """
        Метод для получения разницы между директориями.
        Проверка не пройдена если:
        - Файла или директории не существует
        - Файл имеют разные хеш суммы
        - Файл или папка которая находиться в исключение, но существует в Б

        @return: Список директорий и файлов который нужно скопировать
        """

        def FolderIfNotExist() -> set[str]:
            """
            Получить список папок которых нет в Б директории
            """
            _res_f = set()
            for _path in arr_folder_in:
                _out_path = f"{outfolder}{_path}"
                if not path.isdir(_out_path):
                    _res_f.add(_path)
            return _res_f

        def FileIfChangeDataOrNotExist() -> tuple[set[str], set[str]]:
            """
            Скопировать файл если его нет или, у них различная хеш сумма
            """
            _diff_data_arr_file = set()
            _not_exist_arr_file = set()
            for _path in arr_file_in:
                _in_path = f"{in_folder}{_path}"
                _out_path = f"{outfolder}{_path}"
                # Если файла нет, то копируем его
                if not path.isfile(_out_path):
                    _not_exist_arr_file.add(_path)
                # Если файл уже есть, то проверим его хеш
                else:
                    # Если хеш одинаковый, то не копируем файл
                    # Если хеш разный, то копируем файл
                    if sha256sum(_in_path) != sha256sum(_out_path):
                        _diff_data_arr_file.add(_path)

            return _diff_data_arr_file, _not_exist_arr_file

        """
        Получаем данные о директории А
        """
        # Файлы, которые имеют разную хеш сумму. Файлы которых есть в А, но нет в Б
        diff_data_arr_file, not_exist_arr_file = FileIfChangeDataOrNotExist()
        # Папки которых есть в А, но нет в Б
        not_exist_arr_folder: set[str] = FolderIfNotExist()

        """
        Получаем данные о директории Б 
        """
        # Файлы, которые существуют только в Б.
        # Это может происходить если мы добавили в исключение файл которые ранее в нем не был.
        file_intruder: set[str] = arr_file_out.difference(arr_file_in)
        # Папки, которые существуют только в Б.
        # Это может происходить если мы добавили в исключение файл которые ранее в нем не был.
        folder_intruder: set[str] = arr_folder_out.difference(arr_folder_in)
        _res = DiffDir(in_folder, outfolder,
                       not_exist_arr_file,
                       not_exist_arr_folder,
                       diff_data_arr_file,
                       file_intruder,
                       folder_intruder)
        return _res

    def getDiff(self, arr_exclude_self, folder_two: BaseFolder, arr_exclude_two: set[str]) -> DiffDir:
        """
        Получить различия между дирекцией А и Б

        А - директория откуда брать данные
        Б - директория куда копировать данные
        """
        # Получаем пути к файлам и папкам из директории А
        arr_file_in, arr_folder_in = self.excludeFolderAndFile(arr_exclude_self,
                                                               **self.getAllFileAndFolder())
        # Получаем пути к файлам и папкам из директории Б
        arr_file_out, arr_folder_out = folder_two.excludeFolderAndFile(arr_exclude_two,
                                                                       **folder_two.getAllFileAndFolder())

        # logger.debug(
        #     "Tracking:\n{0}".format(pformat(
        #         {'arr_file_in': arr_file_in,
        #          'arr_folder_in': arr_folder_in,
        #          'arr_file_out': arr_file_out,
        #          'arr_folder_out': arr_folder_out}, compact=True)))

        # Получим разницу между А и Б директориями
        objDiffDir = self._dirDiff(
            self.folder,
            arr_file_in, arr_folder_in,
            folder_two.folder,
            arr_file_out, arr_folder_out,
        )
        return objDiffDir



import unittest
from os.path import getsize
from typing import List, Dict, Any

from mg_file import JsonFile


class TestJson(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        # Данные для теста
        self.testlist: list[list[int | float | str] | dict[int | float | str | Any, int | float | Any]] = [
            [1, 2.1, -1, -2.1, "1", "\t", "Qwe", "Фыв"],
            {12: 2, 1: 1, 1.2: 1.3, 13: 1.2, 4.2: 1, -12: 1, 41: -23, -23.1: -2.2, -232.2: 1,
             "Qwe": 1, 15: "Qwe", -21: "Qwe", 12.3: "DewW", -11: "wasd", "quests": -123},
        ]
        self.name_file = "test.json"

    # Этот метод запускаетсья ПЕРЕД каждой функции теста
    def setUp(self) -> None:
        self.testClassJson = JsonFile(self.name_file)
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()

    def test_sizeFile(self):
        # Проверка определение размера файла
        self.testClassJson.writeFile(self.testlist, sort_keys=False)
        self.assertEqual(self.testClassJson.sizeFile(), getsize(self.testClassJson.name_file))

    def test_deleteFile_and_checkExistenceFile(self):
        # Проверка удаление файла
        self.assertEqual(self.testClassJson.checkExistenceFile(), True)
        self.testClassJson.deleteFile()
        self.assertEqual(self.testClassJson.checkExistenceFile(), False)

    def test_writeJsonFile_and_readJsonFile(self):
        # Проверка записи в файл разных структур данных
        # List
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        temples: List = self.testlist[0]
        self.testClassJson.writeFile(temples)
        self.assertEqual(temples, self.testClassJson.readFile())
        # Dict
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        temples: Dict = {str(k): v for k, v in self.testlist[1].items()}  # все ключи должны быть типа str
        self.testClassJson.writeFile(temples)
        self.assertEqual(temples, self.testClassJson.readFile())

    def test_appendJsonListFile(self):
        # Проверка до записи в файл разных структур данных
        # List
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        tempers: List = self.testlist[0]
        self.testClassJson.writeFile(tempers)
        self.testClassJson.appendFile(tempers)
        tempers += tempers
        self.assertEqual(tempers, self.testClassJson.readFile())

        # Dict
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        tempers: Dict = {str(k): v for k, v in self.testlist[1].items()}  # все ключи должны быть типа str
        self.testClassJson.writeFile(tempers)
        self.testClassJson.appendFile(tempers)
        tempers.update(tempers)
        self.assertEqual(tempers, self.testClassJson.readFile())

    def __del__(self):
        self.testClassJson.deleteFile()



import os

from file.env_file import EnvFile


def test_read():
    r = EnvFile("./data_set/__env.env")
    print(r.readAndSetEnv())
    assert os.environ.get("DEBUG") == "True"
    assert os.environ.get("PORT") == "8080"



from pprint import pprint

import sql_raw.async_sql.aiopg_postgres_sql
import sql_raw.async_sql.async_serializer
import sql_raw.async_sql.asyncpg_postgres_sql
import sql_raw.sync_sql.sync_postgres_sql
import sql_raw.sync_sql.sync_serializer
from .base_dataset_test import NAME_DB, HOST, Refresh_TABLE, NAME_TABEL


def refresh_db():
    """
     Если `FATAL: Peer authentication failed for user "postgres"`

     1. Установить или сменить пароль для пользователя `postgres`
         `sudo passwd postgres;`

     2. Изменить способ подключения е БД
         `sudo micro /etc/postgresql/12/main/pg_hba.conf`
         "
         local   all             postgres                                password
         "
     3. Перезагрузить БД
         `sudo systemctl restart  postgresql`
     """
    # Подключаемся к СУБД
    db = sql_raw.sync_sql.sync_postgres_sql.Config(user="postgres", password="root", host=HOST)
    # Удаляем БД
    print(db.wsql(f"DROP DATABASE IF EXISTS {NAME_DB};", autocommit=True))
    # Создаем БД
    db.wsql(f"CREATE DATABASE {NAME_DB};", autocommit=True)

class Test_Sync:

    def setup(self):
        refresh_db()
        # Подключаемся к БД
        self.db = sql_raw.sync_sql.sync_postgres_sql.Config(user="postgres", password="root", database=NAME_DB,
                                                            host=HOST)
        # Создаем таблицу
        self.db.wsql(Refresh_TABLE)

    def test_write(self):
        self.db.wsql(f"""
        INSERT INTO {NAME_TABEL} (id, email, buy)
        VALUES (1, 'dcurrington0@umn.edu', 955),
               (2, 'rhartland1@blog.com', 430),
               (3, 'zkinton2@so-net.ne.jp', 817),
               (4, 'btuison6@themeforest.net', 281),
               (5, 'gczajka4@tinypic.com', 610),
               (6, 'btuison6@themeforest.net', 281),
               (7, 'btuison6@themeforest.net', 281),
               (8, 'aclancy7@tmall.com', 884),
               (9, 'zkinton2@so-net.ne.jp', 817),
               (10, 'ndelaperrelle9@smh.com.au', 523);
        """)
        assert self.db.Rsql(
            f"SELECT * FROM {NAME_TABEL}") == "[(1, 'dcurrington0@umn.edu', '955,00 ₽'),\n (2, 'rhartland1@blog.com', '430,00 ₽'),\n (3, 'zkinton2@so-net.ne.jp', '817,00 ₽'),\n (4, 'btuison6@themeforest.net', '281,00 ₽'),\n (5, 'gczajka4@tinypic.com', '610,00 ₽'),\n (6, 'btuison6@themeforest.net', '281,00 ₽'),\n (7, 'btuison6@themeforest.net', '281,00 ₽'),\n (8, 'aclancy7@tmall.com', '884,00 ₽'),\n (9, 'zkinton2@so-net.ne.jp', '817,00 ₽'),\n (10, 'ndelaperrelle9@smh.com.au', '523,00 ₽')]"

        assert self.db.rsql(f"SELECT * FROM {NAME_TABEL}") == [
            (1, 'dcurrington0@umn.edu', '955,00 ₽'),
            (2, 'rhartland1@blog.com', '430,00 ₽'),
            (3, 'zkinton2@so-net.ne.jp', '817,00 ₽'),
            (4, 'btuison6@themeforest.net', '281,00 ₽'),
            (5, 'gczajka4@tinypic.com', '610,00 ₽'),
            (6, 'btuison6@themeforest.net', '281,00 ₽'),
            (7, 'btuison6@themeforest.net', '281,00 ₽'),
            (8, 'aclancy7@tmall.com', '884,00 ₽'),
            (9, 'zkinton2@so-net.ne.jp', '817,00 ₽'),
            (10, 'ndelaperrelle9@smh.com.au', '523,00 ₽')]

        assert self.db.rsql(f"SELECT * FROM {NAME_TABEL}", tdata=sql_raw.sync_sql.sync_serializer.Efetch.dict_) == [
            {'id': 1, 'email': 'dcurrington0@umn.edu', 'buy': '955,00 ₽'},
            {'id': 2, 'email': 'rhartland1@blog.com', 'buy': '430,00 ₽'},
            {'id': 3, 'email': 'zkinton2@so-net.ne.jp', 'buy': '817,00 ₽'},
            {'id': 4, 'email': 'btuison6@themeforest.net', 'buy': '281,00 ₽'},
            {'id': 5, 'email': 'gczajka4@tinypic.com', 'buy': '610,00 ₽'},
            {'id': 6, 'email': 'btuison6@themeforest.net', 'buy': '281,00 ₽'},
            {'id': 7, 'email': 'btuison6@themeforest.net', 'buy': '281,00 ₽'},
            {'id': 8, 'email': 'aclancy7@tmall.com', 'buy': '884,00 ₽'},
            {'id': 9, 'email': 'zkinton2@so-net.ne.jp', 'buy': '817,00 ₽'},
            {'id': 10, 'email': 'ndelaperrelle9@smh.com.au', 'buy': '523,00 ₽'}]

        assert self.db.Rsql(f"SELECT * FROM {NAME_TABEL}",
                            tdata=sql_raw.sync_sql.sync_serializer.Efetch.namedtuple) == "[_(id=1, email='dcurrington0@umn.edu', buy='955,00 ₽'),\n _(id=2, email='rhartland1@blog.com', buy='430,00 ₽'),\n _(id=3, email='zkinton2@so-net.ne.jp', buy='817,00 ₽'),\n _(id=4, email='btuison6@themeforest.net', buy='281,00 ₽'),\n _(id=5, email='gczajka4@tinypic.com', buy='610,00 ₽'),\n _(id=6, email='btuison6@themeforest.net', buy='281,00 ₽'),\n _(id=7, email='btuison6@themeforest.net', buy='281,00 ₽'),\n _(id=8, email='aclancy7@tmall.com', buy='884,00 ₽'),\n _(id=9, email='zkinton2@so-net.ne.jp', buy='817,00 ₽'),\n _(id=10, email='ndelaperrelle9@smh.com.au', buy='523,00 ₽')]"

        assert self.db.rsql(f"SELECT * FROM {NAME_TABEL}",
                            tdata=sql_raw.sync_sql.sync_serializer.Efetch.one) == (
                   1, 'dcurrington0@umn.edu', '955,00 ₽')


class Test_Async:

    def setup(self):
        refresh_db()
        # Подключаемся к БД
        self.db = sql_raw.async_sql.asyncpg_postgres_sql.Config(user="postgres", password="root", database=NAME_DB)
        # Создаем таблицу
        self.db.appendTask(self.db.wsql(Refresh_TABLE))
        self.db.executeTasks()

    def test_write(self):
        self.db.appendTask(self.db.wsql(f"""
         INSERT INTO {NAME_TABEL} (id, email, buy)
         VALUES (1, 'dcurrington0@umn.edu', 955),
                (2, 'rhartland1@blog.com', 430),
                (3, 'zkinton2@so-net.ne.jp', 817),
                (4, 'btuison6@themeforest.net', 281),
                (5, 'gczajka4@tinypic.com', 610),
                (6, 'btuison6@themeforest.net', 281),
                (7, 'btuison6@themeforest.net', 281),
                (8, 'aclancy7@tmall.com', 884),
                (9, 'zkinton2@so-net.ne.jp', 817),
                (10, 'ndelaperrelle9@smh.com.au', 523);
         """, isTransaction=True))
        self.db.executeTasks()
        self.db.extendTask(
            [self.db.Rsql(f"SELECT * FROM {NAME_TABEL};"), self.db.Rsql(f"SELECT * FROM {NAME_TABEL};", )])
        assert self.db.executeTasks() == [
            "[<Record id=1 email='dcurrington0@umn.edu' buy='955,00 ₽'>,\n <Record id=2 email='rhartland1@blog.com' buy='430,00 ₽'>,\n <Record id=3 email='zkinton2@so-net.ne.jp' buy='817,00 ₽'>,\n <Record id=4 email='btuison6@themeforest.net' buy='281,00 ₽'>,\n <Record id=5 email='gczajka4@tinypic.com' buy='610,00 ₽'>,\n <Record id=6 email='btuison6@themeforest.net' buy='281,00 ₽'>,\n <Record id=7 email='btuison6@themeforest.net' buy='281,00 ₽'>,\n <Record id=8 email='aclancy7@tmall.com' buy='884,00 ₽'>,\n <Record id=9 email='zkinton2@so-net.ne.jp' buy='817,00 ₽'>,\n <Record id=10 email='ndelaperrelle9@smh.com.au' buy='523,00 ₽'>]",
            "[<Record id=1 email='dcurrington0@umn.edu' buy='955,00 ₽'>,\n <Record id=2 email='rhartland1@blog.com' buy='430,00 ₽'>,\n <Record id=3 email='zkinton2@so-net.ne.jp' buy='817,00 ₽'>,\n <Record id=4 email='btuison6@themeforest.net' buy='281,00 ₽'>,\n <Record id=5 email='gczajka4@tinypic.com' buy='610,00 ₽'>,\n <Record id=6 email='btuison6@themeforest.net' buy='281,00 ₽'>,\n <Record id=7 email='btuison6@themeforest.net' buy='281,00 ₽'>,\n <Record id=8 email='aclancy7@tmall.com' buy='884,00 ₽'>,\n <Record id=9 email='zkinton2@so-net.ne.jp' buy='817,00 ₽'>,\n <Record id=10 email='ndelaperrelle9@smh.com.au' buy='523,00 ₽'>]"]


def test_test():
    Test_Sync().setup()
    db = sql_raw.sync_sql.sync_postgres_sql.Config(user="postgres", password="root", database=NAME_DB, host=HOST)
    db.wsql(f"""
    INSERT INTO {NAME_TABEL} (id, email, buy)
    VALUES (1, 'dcurrington0@umn.edu', 955),
           (2, 'rhartland1@blog.com', 430),
           (3, 'zkinton2@so-net.ne.jp', 817),
           (4, 'btuison6@themeforest.net', 281),
           (5, 'gczajka4@tinypic.com', 610),
           (6, 'btuison6@themeforest.net', 281),
           (7, 'btuison6@themeforest.net', 281),
           (8, 'aclancy7@tmall.com', 884),
           (9, 'zkinton2@so-net.ne.jp', 817),
           (10, 'ndelaperrelle9@smh.com.au', 523);
    """)
    ###
    db = sql_raw.async_sql.async_postgres_sql.Config(user="postgres", password="root", database=NAME_DB, host=HOST)
    db.extendTask([db.test(f"SELECT * FROM {NAME_TABEL};"),  # db.test(f"SELECT * FROM {NAME_TABEL};"),
                   # db.test(f"SELECT * FROM {NAME_TABEL};"),
                   # db.rsql(f"SELECT * FROM {NAME_TABEL};"),
                   # db.rsql(f"SELECT id FROM {NAME_TABEL};"),
                   # db.Rsql(f"SELECT * FROM {NAME_TABEL};"),
                   # db.Rsql(f"SELECT * FROM {NAME_TABEL};"),
                   ])
    pprint(db.executeTasks())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mg_file',
 'mg_file.castom_type',
 'mg_file.file',
 'mg_file.file.test',
 'mg_file.sql_raw',
 'mg_file.sql_raw.async_sql',
 'mg_file.sql_raw.sync_sql',
 'mg_file.sql_raw.test',
 'mg_file.sqllite_orm',
 'mg_file.sqllite_orm.test']

package_data = \
{'': ['*'], 'mg_file.file.test': ['data_set/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'loguru>=0.6.0,<0.7.0',
 'prettytable>=3.0.0,<4.0.0',
 'typing-extensions>=4.1.1,<5.0.0']

setup_kwargs = {
    'name': 'mg-file',
    'version': '0.1.10.3',
    'description': 'Удобное пользование файлами',
    'long_description': '# Что это\n\nБиблиотека для работы с файлами и СУБД\n\n# Установка через [`pip`](https://pypi.org/project/mg-file/)\n\n```bash\npip install mg-file\n```\n\n# Технологии\n\n| Технологии                                      | Описание                                                      |\n|-------------------------------------------------|---------------------------------------------------------------|\n| [Файлы](mg_file/file/README.md)                 | Работа с `txt`, `csv` , `json` , `pickl`                      |\n| [Чистый SQL запросы](mg_file/sql_raw/README.md) | Синхронный / Асинхронный доступ к СУБД `PostgreSQL` , `MySql` |\n| [SQL ORM](mg_file/sqllite_orm/README.md)        | `ORM` система                                                 |\n\n\n',
    'author': 'Denis Vetkin',
    'author_email': 'denis-kustov@rambler.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/denisxab/mg_file',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from abc import abstractmethod, ABCMeta
from asyncio import run, gather
from collections import deque
from pprint import pformat
from typing import Any, Coroutine, Callable

from loguru import logger

from .async_serializer import BaseTasks
from ..base_sql import BaseSql


class AsyncBaseSql(BaseSql, BaseTasks, metaclass=ABCMeta):
    """
    Базовый асинхронны класс
    """

    def __init__(self,
                 user: str,
                 password: str,
                 host: str = "localhost"):
        # Создаем настройки
        BaseSql.__init__(self, user, password, host)
        # Создаем список с задач
        BaseTasks.__init__(self)
        self.Poll: Callable | object | None = None
        self.FuncConnectDb: Callable | object | None = None

    # BASE CONNECT
    def executeTasks(self):
        """Запустить выполнения задач"""
        res = run(self._run(self.tasks))
        self.tasks.clear()
        return res

    async def _run(self, tasks: deque[Coroutine]):
        self.FuncConnectDb = self.connect_db
        """Выполнить список задач"""
        return await gather(*tasks)

    async def connect_db(self, fun: Callable, *args, **kwargs) -> Any:
        """
        Так как у нас всегда включен `autocommit` мы
        можем использовать контекстный менеджер `with`
        """
        try:
            async with self.ConnectCallback(**self.SETTINGS_DB) as connection:
                return await fun(connection, *args, **kwargs)
        except self.ERROR as e:
            logger.error(e)
            raise e

    # POOL

    def executeTasksPool(self):
        """Запустить выполнения задач"""
        res = run(self._run_pool(self.tasks))
        self.tasks.clear()
        return res

    async def _run_pool(self, tasks: deque[Coroutine]):
        """Выполнить список задач"""
        self.FuncConnectDb = self.connect_pool
        # Создаем пул соединений
        async with self.ConnectCallback(**self.SETTINGS_DB) as pool:
            # Сохраняем пул в экземпляре
            self.Poll = pool
            # Запускаем запросы
            return await gather(*tasks)

    async def connect_pool(self, fun: Callable, *args, **kwargs):
        try:
            async with self.Poll.acquire() as connection:
                return await fun(connection, *args, **kwargs)
        except self.ERROR as e:
            logger.error(e)
            raise e

    # Abstract
    @property
    @abstractmethod
    def ERROR(self):
        ...

    @property
    @abstractmethod
    def ConnectCallback(self):
        ...


class AsyncBaseCommand(AsyncBaseSql, metaclass=ABCMeta):
    """
    from sql_raw.async_sql.async_postgres_sql import Config
    from sql_raw.async_sql.async_serializer import Efetch

    # Создать конфигурацию
    db = Config(user="postgres", password="root", database="fast_api")

    # Добавить список задач
    db.extendTask([
        db.rsql("SELECT * FROM пользователь;", tdata=Efetch.dict_),
        db.rsql("SELECT id FROM пользователь;"),
        db.rsql("SELECT * FROM пользователь;"),
    ])

    # Добавить одну задачу
    db.appendTask(db.rsql("SELECT * FROM пользователь;"))

    # Выполнить задачи
    pprint(db.executeTasks())
    """

    def __init__(self,
                 user: str,
                 password: str,
                 host: str = "localhost"):
        AsyncBaseSql.__init__(self, user, password, host)

    async def rsql(self, execute: str,
                   params: tuple | dict | list = ()) -> str:
        """
        Чтение из БД
        """
        return await self.FuncConnectDb(self.read_command,
                                        execute=execute,
                                        params=params)

    async def rsql_transaction(self, execute: str,
                               params: tuple | dict | list = ()) -> str:
        """
        Чтение из БД транзакция
        """
        return await self.FuncConnectDb(self.transaction_read_command,
                                        execute=execute,
                                        params=params)

    async def wsql(self, execute: str,
                   params: tuple | dict | list = (),
                   ) -> tuple[str, tuple | dict | list]:
        """
        Внесение изменений в БД
        """
        return await self.FuncConnectDb(self.mutable_command,
                                        execute=execute
                                        , params=params)

    async def wsql_transaction(self, execute: str,
                               params: tuple | dict | list = (),
                               ) -> tuple[str, tuple | dict | list]:
        """
        Внесение изменений в БД
        """
        return await self.FuncConnectDb(self.transaction_mutable_command,
                                        execute=execute,
                                        params=params)

    # Утилиты
    async def Rsql(self, execute: str,
                   params: tuple | dict | list = (),
                   isTransaction: bool = False) -> str:
        """
        Чтение из БД с красивым выводом в консоль
        """
        return await self.pprint_deco(self.rsql,
                                      execute,
                                      params,
                                      isTransaction)

    @staticmethod
    async def pprint_deco(fun: Callable,
                          execute: str,
                          params: tuple | dict | list = (),
                          isTransaction: bool = False
                          ) -> str:
        """
        Декоратор для красивого вывода результата функции в консоль
        """
        return pformat(await fun(execute, params, isTransaction))

    # Abstractmethod
    @abstractmethod
    async def read_command(self, _connection,
                           execute: str,
                           params: tuple | dict | list = (), ):
        """
        Метод для выполнения чтения из БД
        """
        return NotImplemented()

    @abstractmethod
    async def mutable_command(self, _connection,
                              execute: str,
                              params: tuple | dict | list = (), ):
        """
        Метод для выполнения записи в БД
        """
        return NotImplemented()

    @abstractmethod
    async def transaction_mutable_command(self, _connection,
                                          execute: str,
                                          params: tuple | dict | list = (),
                                          ):
        return NotImplemented()

    @abstractmethod
    async def transaction_read_command(self, _connection,
                                       execute: str,
                                       params: tuple | dict | list = (),
                                       ) -> Any:
        return NotImplemented()



import time

import sql_raw.async_sql.aiopg_postgres_sql
import sql_raw.async_sql.async_serializer
import sql_raw.async_sql.asyncpg_postgres_sql
import sql_raw.sync_sql.sync_postgres_sql
import sql_raw.sync_sql.sync_serializer
from .base_dataset_test import NAME_DB, HOST
from .test_postgres import NAME_TABEL, Test_Sync


def test_speed_async(SQL_COMMAND: str, LEN: int):
    start = time.process_time()
    db = sql_raw.async_sql.asyncpg_postgres_sql.Config(user="postgres", password="root", database=NAME_DB, host=HOST)
    db.extendTask([
        db.rsql_transaction(SQL_COMMAND) for _ in range(LEN)
    ])
    res = db.executeTasksPool()
    end = time.process_time()
    print([len(x) for x in res])
    print("asYncpg", end - start)


def test_speed_t_async(SQL_COMMAND: str, LEN: int):
    start = time.process_time()
    db = sql_raw.async_sql.aiopg_postgres_sql.Config(user="postgres", password="root", database=NAME_DB, host=HOST)
    db.extendTask([
        db.rsql(SQL_COMMAND) for _ in range(LEN)
    ])
    res = db.executeTasks()
    end = time.process_time()
    print([len(x) for x in res])
    print("aiOpg", end - start)


def test_speed_sync(SQL_COMMAND: str, LEN: int):
    start = time.process_time()
    db = sql_raw.sync_sql.sync_postgres_sql.Config(user="postgres", password="root", database=NAME_DB, host=HOST)
    res = [
        db.rsql(SQL_COMMAND) for _ in range(LEN)
    ]
    end = time.process_time()
    print([len(x) for x in res])
    print("SYNC:", end - start)


def test_main_speed():
    Test_Sync().setup()
    db = sql_raw.sync_sql.sync_postgres_sql.Config(user="postgres", password="root", database=NAME_DB, host=HOST)
    db.wsql(f"""
     INSERT INTO {NAME_TABEL} (id, email, buy)
     VALUES (1, 'dcurrington0@umn.edu', 955),
            (2, 'rhartland1@blog.com', 430),
            (3, 'zkinton2@so-net.ne.jp', 817),
            (4, 'btuison6@themeforest.net', 281),
            (5, 'gczajka4@tinypic.com', 610),
            (6, 'btuison6@themeforest.net', 281),
            (7, 'btuison6@themeforest.net', 281),
            (8, 'aclancy7@tmall.com', 884),
            (9, 'zkinton2@so-net.ne.jp', 817),
            (10, 'ndelaperrelle9@smh.com.au', 523);
     """)
    SQL_COMMAND = f"SELECT * FROM {NAME_TABEL}"
    print("START")
    LEN = 100
    test_speed_async(SQL_COMMAND, LEN)
    test_speed_t_async(SQL_COMMAND, LEN)
    test_speed_sync(SQL_COMMAND, LEN)
